from dataclasses import dataclass, field
from typing import Any, List

from netspresso.clients.compressor.schemas.model import InputLayer, ModelResponse


@dataclass
class InputShape:
    """Represents the shape of an input tensor.

    Attributes:
        batch (int): The batch size of the input tensor.
        channel (int): The number of channels in the input tensor.
        dimension (List[int]): The dimensions of the input tensor.
    """

    batch: int
    channel: int
    dimension: List[int]


@dataclass
class Model:
    """Represents a uploaded model.

    Attributes:
        model_id (str): The ID of the model.
        model_name (str): The name of the model.
        task (Task): The task of the model.
        framework (Framework): The framework of the model.
        input_shapes (List[InputShape]): The input shapes of the model.

            InputShape Attributes:
                - batch (int): The batch size of the input tensor.
                - channel (int): The number of channels in the input tensor.
                - dimension (List[int]): The dimensions of the input tensor.

        model_size (float): The size of the model.
        flops (float): The FLOPs (floating point operations) of the model.
        trainable_parameters (float): The number of trainable parameters in the model.
        non_trainable_parameters (float): The number of non-trainable parameters in the model.
        number_of_layers (float): The number of layers in the model.
    """

    model_id: str
    model_name: str
    task: str
    framework: str
    model_size: float
    flops: float
    trainable_parameters: float
    non_trainable_parameters: float
    number_of_layers: int
    input_shapes: List[InputShape] = field(default_factory=list)

    def set_input_shapes(self, input_layers):
        self.input_shapes = [InputShape(**layer.dict()) for layer in input_layers]


@dataclass
class CompressedModel(Model):
    """Represents a compressed model.

    Attributes:
        compression_id (str): The ID of the compression.
        original_model_id (str): The ID of the uploaded model.
    """

    compression_id: str = ""
    original_model_id: str = ""


@dataclass
class ModelCollection(Model):
    """A collection of models that includes the uploaded model and its compressed models.

    Attributes:
        compressed_models (List[CompressedModel]): A list of compressed models compressed from this model.
    """

    compressed_models: List[CompressedModel] = field(default_factory=list)

    def set_compressed_models(self, children_models):
        for children_model_info in children_models:
            compressed_model = CompressedModel(
                model_id=children_model_info.model_id,
                model_name=children_model_info.model_name,
                task=children_model_info.task,
                framework=children_model_info.framework,
                model_size=children_model_info.spec.model_size,
                flops=children_model_info.spec.flops,
                trainable_parameters=children_model_info.spec.trainable_parameters,
                non_trainable_parameters=children_model_info.spec.non_trainable_parameters,
                number_of_layers=children_model_info.spec.number_of_layers,
                compression_id=children_model_info.original_compression_id,
                original_model_id=children_model_info.original_model_id,
            )
            compressed_model.set_input_shapes(input_layers=children_model_info.spec.input_layers)
            self.compressed_models.append(compressed_model)


class ModelFactory:
    @staticmethod
    def extract_model_attributes(model_info: ModelResponse):
        return (
            model_info.model_id,
            model_info.model_name,
            model_info.task,
            model_info.framework,
            model_info.spec.model_size,
            model_info.spec.flops,
            model_info.spec.trainable_parameters,
            model_info.spec.non_trainable_parameters,
            model_info.spec.number_of_layers,
        )

    def create_model(self, model_info: ModelResponse) -> Model:
        attributes = self.extract_model_attributes(model_info)
        model = Model(*attributes)
        model.set_input_shapes(input_layers=model_info.spec.input_layers)

        return model

    def create_compressed_model(self, model_info: ModelResponse) -> CompressedModel:
        attributes = self.extract_model_attributes(model_info)
        compressed_model = CompressedModel(
            *attributes,
            compression_id=model_info.original_compression_id,
            original_model_id=model_info.original_model_id
        )
        compressed_model.set_input_shapes(input_layers=model_info.spec.input_layers)

        return compressed_model

    def create_model_collection(
        self, model_info: ModelResponse, children_models: List[ModelResponse]
    ) -> ModelCollection:
        attributes = self.extract_model_attributes(model_info)
        model_collection = ModelCollection(*attributes)
        model_collection.set_input_shapes(input_layers=model_info.spec.input_layers)
        model_collection.set_compressed_models(children_models=children_models)

        return model_collection



from pathlib import Path

from netspresso.clients.utils.system import ENV_STR

version = (Path(__file__).parent.parent.parent / "VERSION").read_text().strip()


def get_headers(access_token=None, json_type=False):
    headers = {"User-Agent": f"NetsPresso Python Package v{version} ({ENV_STR})"}
    if access_token:
        headers["Authorization"] = f"Bearer {access_token}"
    if json_type:
        headers["Content-Type"] = "application/json"
    return headers


def create_tao_headers(token):
    return {"Authorization": f"Bearer {token}"}


def read_file_bytes(file_path: str) -> bytes:
    with open(file_path, "rb") as f:
        file_byte = f.read()
    return file_byte


def get_files(file_path):
    return [
        (
            "file",
            (Path(file_path).name, open(file_path, "rb"), "application/octet-stream"),
        )
    ]



import time
from pathlib import Path
from typing import Dict, Optional, Union
from urllib import request

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.schemas.auth import UserInfo
from netspresso.clients.launcher import launcher_client
from netspresso.clients.launcher.schemas import TargetDeviceFilter
from netspresso.clients.launcher.schemas.model import ConversionTask, InputShape, Model
from netspresso.enums import (
    DataType,
    DeviceName,
    Framework,
    Module,
    ServiceCredit,
    SoftwareVersion,
    Status,
    TaskStatus,
    TaskType,
)

from ..utils import FileHandler, check_credit_balance
from ..utils.metadata import MetadataHandler


class Converter:
    def __init__(self, token_handler: TokenHandler, user_info: UserInfo):
        """Initialize the Converter."""

        self.token_handler = token_handler
        self.user_info = user_info

    def _download_converted_model(self, conversion_task: ConversionTask, local_path: str) -> None:
        """Download the converted model with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConversionTask): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.
        """

        try:
            if conversion_task.status is TaskStatus.ERROR:
                raise FileNotFoundError("The conversion is Failed. There is no file available for download.")
            if conversion_task.status is not TaskStatus.FINISHED:
                raise FileNotFoundError(
                    "The conversion is in progress. There is no file available for download at the moment."
                )

            download_url = launcher_client.get_converted_model(
                conversion_task_uuid=conversion_task.convert_task_uuid,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            request.urlretrieve(download_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download converted model failed. Error: {e}")
            raise e

    def convert_model(
        self,
        input_model_path: str,
        output_dir: str,
        target_framework: Union[str, Framework],
        target_device_name: Union[str, DeviceName],
        target_data_type: Union[str, DataType] = DataType.FP16,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        input_shape: Optional[InputShape] = None,
        dataset_path: Optional[str] = None,
        wait_until_done: bool = True,
    ) -> Dict:
        """Convert a model to the specified framework.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the converted model.
            target_framework (Union[str, Framework]): The target framework name.
            target_device_name (Union[str, DeviceName]): Target device name. Required if target_device is not specified.
            target_data_type (Union[str, DataType]): Data type of the model. Default is DataType.FP16.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version.
                Required if target_device_name is one of the Jetson devices.
            input_shape (InputShape, optional): Target input shape for conversion (e.g., dynamic batch to static batch).
            dataset_path (str, optional): Path to the dataset. Useful for certain conversions.
            wait_until_done (bool): If True, wait for the conversion result before returning the function.
                                If False, request the conversion and return the function immediately.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            Dict: Model conversion task dictionary.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        try:
            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=target_framework
            )
            FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = MetadataHandler.init_metadata(folder_path=output_dir, task_type=TaskType.CONVERT)

            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            check_credit_balance(user_credit=current_credit, service_credit=ServiceCredit.MODEL_CONVERT)
            model = launcher_client.upload_model(
                model_file_path=input_model_path,
                target_function=Module.CONVERT,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            model_uuid = model
            if type(model) is Model:
                model_uuid = model.model_uuid
                if input_shape is None and model.input_shape is not None:
                    input_shape = model.input_shape
                if target_framework is None and model.framework is not None:
                    target_framework = model.framework

            elif target_device_name in DeviceName.JETSON_DEVICES and target_software_version is None:
                raise NotImplementedError(
                    "The conversion is unavailable. Please set JetPack version with target_software_version for Jetson Devices."
                )

            if type(model) is not Model:
                raise NotImplementedError(
                    "The conversion is unavailable. Please set target_device while using model's uuid string for the conversion."
                )

            # Check available int8 converting devices
            if target_data_type == DataType.INT8:
                if target_device_name not in DeviceName.AVAILABLE_INT8_DEVICES:
                    raise Exception(f"int8 converting supports only {DeviceName.AVAILABLE_INT8_DEVICES}.")
            else:  # FP16, FP32
                if target_device_name in DeviceName.ONLY_INT8_DEVICES:
                    raise Exception(f"{DeviceName.ONLY_INT8_DEVICES} only support int8 data types.")

            devices = TargetDeviceFilter.filter_devices_with_device_name(
                name=target_device_name, devices=model.available_devices
            )

            if target_device_name in DeviceName.JETSON_DEVICES:
                devices = TargetDeviceFilter.filter_devices_with_device_software_version(
                    software_version=target_software_version, devices=devices
                )

            if len(devices) < 1:
                raise NotImplementedError(
                    "The conversion is unavailable. There is no available device with given target_device_name and target_software_version."
                )

            target_device = devices[0]

            logger.info(f"Converting Model for {target_device.device_name} ({target_framework})")

            conversion_task = launcher_client.convert_model(
                user_uuid=self.user_info.user_id,
                model_uuid=model_uuid,
                input_shape=input_shape,
                target_framework=target_framework,
                target_device=target_device.device_name,
                data_type=target_data_type,
                software_version=target_device.software_version,
                dataset_path=dataset_path,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            conversion_task = self.get_conversion_task(conversion_task)

            if wait_until_done:
                while conversion_task.status in [
                    TaskStatus.IN_QUEUE,
                    TaskStatus.IN_PROGRESS,
                ]:
                    conversion_task = self.get_conversion_task(conversion_task)
                    time.sleep(1)

            self._download_converted_model(conversion_task, default_model_path.with_suffix(extension))

            converter_uploaded_model = launcher_client.upload_model(
                model_file_path=default_model_path.with_suffix(extension),
                target_function=Module.BENCHMARK,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            metadata.update_converted_model_path(
                converted_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            metadata.update_model_info(
                data_type=model.data_type,
                framework=model.framework,
                input_shape=model.input_shape,
            )
            metadata.update_convert_info(
                target_framework=conversion_task.target_framework,
                target_device_name=conversion_task.target_device_name,
                data_type=conversion_task.data_type,
                software_version=conversion_task.software_version,
                model_file_name=conversion_task.model_file_name,
                convert_task_uuid=conversion_task.convert_task_uuid,
                input_model_uuid=conversion_task.input_model_uuid,
                output_model_uuid=conversion_task.output_model_uuid,
            )
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_devices(converter_uploaded_model.available_devices)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.MODEL_CONVERT} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

            return metadata.asdict()

        except Exception as e:
            logger.error(f"Convert failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def get_conversion_task(self, conversion_task: Union[str, ConversionTask]) -> ConversionTask:
        """Get the conversion task information with given conversion task or conversion task uuid.

        Args:
            conversion_task (Union[str, ConversionTask]): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            ConversionTask: Model conversion task dictionary.
        """

        self.token_handler.validate_token()

        try:
            conversion_task_uuid = None
            if isinstance(conversion_task, str):
                conversion_task_uuid = conversion_task
            elif type(conversion_task) is ConversionTask:
                conversion_task_uuid = conversion_task.convert_task_uuid
            else:
                raise NotImplementedError(
                    "There is no available function for the given parameter. The 'conversion_task' should be a UUID string or a ModelConversion object."
                )
            return launcher_client.get_conversion_task(
                conversion_task_uuid=conversion_task_uuid,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

        except Exception as e:
            logger.error(f"Get conversion task failed. Error: {e}")
            raise e



import json
import shutil
import sys
from pathlib import Path
from typing import Tuple, Union
from urllib import request

FRAMEWORK_EXTENSION_MAP = {
    "tensorflow_keras": ".h5",
    "pytorch": ".pt",
    "onnx": ".onnx",
    "tensorflow_lite": ".tflite",
    "drpai": ".zip",
    "openvino": ".zip",
    "tensorrt": ".trt",
}


class FileHandler:
    """Utility class for file-related operations."""

    @staticmethod
    def check_exists(folder_path: str) -> bool:
        """Check if the file or folder exists.

        Args:
            folder_path (str): The path to the folder.

        Returns:
            bool: True if the file or folder exists, False otherwise
        """
        return Path(folder_path).exists()

    @staticmethod
    def check_input_model_path(input_model_path: str):
        """Check if the input model path is a file.

        Args:
            input_model_path (str): The path to the input model file.

        Raises:
            FileNotFoundError: If the input model path is not a file.
        """

        if not Path(input_model_path).is_file():
            raise FileNotFoundError(
                "The input_model_path should be a file and cannot be a directory. Ex) ./model/sample_model.pt"
            )

    @staticmethod
    def create_folder(
        folder_path: str,
        parents: bool = True,
        exist_ok: bool = True,
        is_folder_check: bool = True,
    ) -> None:
        """Create a folder.

        Args:
            folder_path (str): The path to the folder to be created.
            parents (bool, optional): If True, also create parent directories if they don't exist.
            exist_ok (bool, optional): If False, raise an error if the folder already exists.
            is_folder_check (bool, optional): If True, check if the folder already exists.

        Raises:
            SystemExit: If the folder already exists and `exist_ok` is False.
        """
        if is_folder_check and not FileHandler.check_exists(folder_path=folder_path):
            Path(folder_path).mkdir(parents=parents, exist_ok=exist_ok)
        elif is_folder_check:
            sys.exit(f"This folder already exists. Local Path: {Path(folder_path)}")

    @staticmethod
    def create_unique_folder(folder_path: str) -> None:
        folder_path = Path(folder_path)
        if not folder_path.exists():
            folder_path.mkdir(parents=True)
        else:
            count = 1
            while True:
                new_folder_path = folder_path.with_name(f"{folder_path.name} ({count})")
                if not new_folder_path.exists():
                    new_folder_path.mkdir(parents=True)
                    break
                count += 1

    @staticmethod
    def create_file_path(folder_path: str, name: str, extension: str) -> Union[str, Path]:
        """Create a file path.

        Args:
            folder_path (str): The path to the folder where the file will be located.
            name (str): The name of the file.
            extension (str): The file extension.

        Returns:
            Union[str, Path]: The full path to the created file.
        """
        return Path(folder_path) / (name + extension)

    @staticmethod
    def download_file(url: str, save_path: Union[str, Path]) -> None:
        """Download a file from the given URL and save it to the specified path.

        Args:
            url (str): The URL of the file to be downloaded.
            save_path (Union[str, Path]): The path where the downloaded file will be saved.
        """
        request.urlretrieve(url, save_path)

    @staticmethod
    def get_extension_by_framework(framework: str) -> str:
        """Get the file extension based on the given framework.

        Args:
            framework (str): The framework name.

        Raises:
            KeyError: If the framework is not found in the extension map.

        Returns:
            str: The file extension corresponding to the framework.
        """
        extension = FRAMEWORK_EXTENSION_MAP.get(framework)
        if extension is None:
            available_frameworks = list(FRAMEWORK_EXTENSION_MAP.keys())
            raise KeyError(f"The framework supports {available_frameworks}. The entered framework is {framework}.")
        return extension

    @staticmethod
    def get_path_and_extension(folder_path: str, framework: str) -> Tuple[Path, str]:
        """Prepare the model path by creating folders and generating a default model path.

        Args:
            folder_path (str): The base folder path.
            framework (str): The framework name.
            is_folder_check (bool, optional): If True, check if the folder exists before creating.

        Returns:
            Tuple[Path, str]: A tuple containing the default model path (Path) and the file extension (str).
        """
        default_model_path = Path(folder_path) / f"{Path(folder_path).name}.ext"
        extension = FileHandler.get_extension_by_framework(framework=framework)

        return default_model_path, extension

    @staticmethod
    def load_json(file_path: str):
        """Load JSON data from a file.

        Args:
            file_path (str): Path to the JSON file.

        Returns:
            dict: Loaded JSON data.
        """

        with open(file_path, "r") as json_data:
            data = json.load(json_data)
        return data

    @staticmethod
    def move_and_cleanup_folders(source_folder: str, destination_folder: str):
        """Move files from the source folder to the destination folder and remove the source folder.

        Args:
            source_folder (str): The path to the source folder.
            destination_folder (str): The path to the destination folder.
        """

        source_folder = Path(source_folder)
        destination_folder = Path(destination_folder)

        for file_path in source_folder.iterdir():
            destination_path = destination_folder / file_path.name
            shutil.move(file_path, destination_path)

        source_folder.rmdir()

    @staticmethod
    def remove_folder(folder_path: str) -> None:
        """Remove a folder and its contents.

        Args:
            folder_path (str): Path to the folder.
        """

        folder_path = Path(folder_path)
        shutil.rmtree(folder_path, ignore_errors=True)



from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from loguru import logger
from netspresso_trainer import train_with_yaml
from netspresso_trainer.cfg import AugmentationConfig, EnvironmentConfig, LoggingConfig, ModelConfig, ScheduleConfig
from netspresso_trainer.cfg.augmentation import Inference, Train, Transform
from netspresso_trainer.cfg.data import ImageLabelPathConfig, PathConfig
from netspresso_trainer.cfg.model import CheckpointConfig
from omegaconf import OmegaConf

from netspresso.enums import Status, Task, TaskType

from ..utils import FileHandler
from ..utils.metadata import MetadataHandler
from ..utils.metadata.default.trainer import InputShape
from .registries import (
    AUGMENTATION_CONFIG_TYPE,
    CLASSIFICATION_MODELS,
    DATA_CONFIG_TYPE,
    DETECTION_MODELS,
    SEGMENTATION_MODELS,
    TRAINING_CONFIG_TYPE,
)
from .trainer_configs import TrainerConfigs


class Trainer:
    def __init__(self, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None) -> None:
        """Initialize the Trainer.

        Args:
            task (Union[str, Task]], optional): The type of task (classification, detection, segmentation). Either 'task' or 'yaml_path' must be provided, but not both.
            yaml_path (str, optional): Path to the YAML configuration file. Either 'task' or 'yaml_path' must be provided, but not both.
        """

        if (task is not None) == (yaml_path is not None):
            raise ValueError("Either 'task' or 'yaml_path' must be provided, but not both.")

        if task is not None:
            self._initialize_from_task(task)
        elif yaml_path is not None:
            self._initialize_from_yaml(yaml_path)

    def _initialize_from_task(self, task: Union[str, Task]) -> None:
        """Initialize the Trainer object based on the provided task.

        Args:
            task (Union[str, Task]): The task for which the Trainer is initialized.
        """

        self.task = self._validate_task(task)
        self.available_models = list(self._get_available_models().keys())
        self.data = None
        self.model = None
        self.training = TRAINING_CONFIG_TYPE[self.task]()
        self.augmentation = AUGMENTATION_CONFIG_TYPE[self.task]()
        self.logging = LoggingConfig()
        self.environment = EnvironmentConfig()

    def _initialize_from_yaml(self, yaml_path: str) -> None:
        """Initialize the Trainer object based on the configuration provided in a YAML file.

        Args:
            yaml_path (str): The path to the YAML file containing the configuration.
        """

        hparams = OmegaConf.load(yaml_path)
        hparams["model"].pop("single_task_model")

        self.img_size = hparams["augmentation"]["img_size"]
        self.task = hparams["data"]["task"]
        self.available_models = list(self._get_available_models().keys())

        self.data = DATA_CONFIG_TYPE[self.task](**hparams["data"])
        self.model = ModelConfig(**hparams["model"])
        self.training = ScheduleConfig(**hparams["training"])
        self.augmentation = AugmentationConfig(**hparams["augmentation"])
        self.logging = LoggingConfig(**hparams["logging"])
        self.environment = EnvironmentConfig(**hparams["environment"])

    def _validate_task(self, task: Union[str, Task]):
        """Validate the provided task.

        Args:
            task (Union[str, Task]): The task to be validated.

        Raises:
            ValueError: If the provided task is not supported.

        Returns:
            Task: The validated task.
        """

        available_tasks = [task.value for task in Task]
        if task not in available_tasks:
            raise ValueError(f"The task supports {available_tasks}. The entered task is {task}.")
        return task

    def _validate_config(self):
        """Validate the configuration setup.

        Raises:
            ValueError: Raised if the dataset is not set. Use `set_dataset_config` or `set_dataset_config_with_yaml` to set the dataset configuration.
            ValueError: Raised if the model is not set. Use `set_model_config` or `set_model_config_with_yaml` to set the model configuration.
        """

        if self.data is None:
            raise ValueError(
                "The dataset is not set. Use `set_dataset_config` or `set_dataset_config_with_yaml` to set the dataset configuration."
            )
        if self.model is None:
            raise ValueError(
                "The model is not set. Use `set_model_config` or `set_model_config_with_yaml` to set the model configuration."
            )

    def _get_available_models(self) -> Dict[str, Any]:
        """Get available models based on the current task.

        Returns:
            Dict[str, Any]: A dictionary mapping model types to available models.
        """

        available_models = {
            "classification": CLASSIFICATION_MODELS,
            "detection": DETECTION_MODELS,
            "segmentation": SEGMENTATION_MODELS,
        }[self.task]

        return available_models

    def set_dataset_config(
        self,
        name: str,
        root_path: str,
        train_image: str = "images/train",
        train_label: str = "labels/train",
        valid_image: str = "images/val",
        valid_label: str = "labels/val",
        id_mapping: Optional[Union[List[str], Dict[str, str]]] = None,
    ):
        """Set the dataset configuration for the Trainer.

        Args:
            name (str): The name of dataset.
            root_path (str): Root directory of dataset.
            train_image (str, optional): The directory for training images. Should be relative path to root directory. Defaults to "images/train".
            train_label (str, optional): The directory for training labels. Should be relative path to root directory. Defaults to "labels/train".
            valid_image (str, optional): The directory for validation images. Should be relative path to root directory. Defaults to "images/val".
            valid_label (str, optional): The directory for validation labels. Should be relative path to root directory. Defaults to "labels/val".
            id_mapping (Union[List[str], Dict[str, str]], optional): ID mapping for the dataset. Defaults to None.
        """

        common_config = {
            "name": name,
            "path": PathConfig(
                root=root_path,
                train=ImageLabelPathConfig(image=train_image, label=train_label),
                valid=ImageLabelPathConfig(image=valid_image, label=valid_label),
            ),
            "id_mapping": id_mapping,
        }
        self.data = DATA_CONFIG_TYPE[self.task](**common_config)

    def set_model_config(
        self,
        model_name: str,
        img_size: int,
        use_pretrained: bool = True,
        load_head: bool = False,
        path: Optional[str] = None,
        fx_model_path: Optional[str] = None,
        optimizer_path: Optional[str] = None,
    ):
        """Set the model configuration for the Trainer.

        Args:
            model_name (str): Name of the model.
            img_size (int): Image size for the model.
            use_pretrained (bool, optional): Whether to use a pre-trained model. Defaults to True.
            load_head (bool, optional): Whether to load the model head. Defaults to False.
            path (str, optional): Path to the model. Defaults to None.
            fx_model_path (str, optional): Path to the FX model. Defaults to None.
            optimizer_path (str, optional): Path to the optimizer. Defaults to None.

        Raises:
            ValueError: If the specified model is not supported for the current task.
        """

        model = self._get_available_models().get(model_name)
        self.img_size = img_size

        if model is None:
            raise ValueError(
                f"The '{model_name}' model is not supported for the '{self.task}' task. The available models are {self.available_models}."
            )

        self.model = model(
            checkpoint=CheckpointConfig(
                use_pretrained=use_pretrained,
                load_head=load_head,
                path=path,
                fx_model_path=fx_model_path,
                optimizer_path=optimizer_path,
            )
        )

    def set_fx_model(self, fx_model_path: str):
        """Set the FX model path for retraining.

        Args:
            fx_model_path (str): The path to the FX model.

        Raises:
            ValueError: If the model is not set. Please use 'set_model_config' for model setup.
        """

        if not self.model:
            raise ValueError("This function is intended for retraining. Please use 'set_model_config' for model setup.")

        self.model.checkpoint.path = None
        self.model.checkpoint.fx_model_path = fx_model_path

    def set_training_config(
        self,
        optimizer,
        scheduler,
        epochs: int = 3,
        batch_size: int = 8,
    ):
        """Set the training configuration.

        Args:
            optimizer: The configuration of optimizer.
            scheduler: The configuration of learning rate scheduler.
            epochs (int, optional): The total number of epoch for training the model. Defaults to 3.
            batch_size (int, optional): The number of samples in single batch input. Defaults to 8.
        """

        self.training = ScheduleConfig(
            epochs=epochs,
            batch_size=batch_size,
            optimizer=optimizer.asdict(),
            scheduler=scheduler.asdict(),
        )

    def set_augmentation_config(
        self,
        train_transforms: Optional[List] = None,
        train_mix_transforms: Optional[List] = None,
        inference_transforms: Optional[List] = None,
    ):
        """Set the augmentation configuration for training.

        Args:
            train_transforms (List, optional): List of transforms for training. Defaults to None.
            train_mix_transforms (List, optional): List of mix transforms for training. Defaults to None.
            inference_transforms (List, optional): List of transforms for inference. Defaults to None.
        """

        self.augmentation = AugmentationConfig(
            train=Train(
                transforms=train_transforms,
                mix_transforms=train_mix_transforms,
            ),
            inference=Inference(
                transforms=inference_transforms,
            ),
        )

    def set_logging_config(
        self,
        project_id: Optional[str] = None,
        output_dir: str = "./outputs",
        tensorboard: bool = True,
        csv: bool = False,
        image: bool = True,
        stdout: bool = True,
        save_optimizer_state: bool = True,
        validation_epoch: int = 10,
        save_checkpoint_epoch: Optional[int] = None,
    ):
        """Set the logging configuration.

        Args:
            project_id (str, optional): Project name to save the experiment. If None, it is set as {task}_{model} (e.g. segmentation_segformer).
            output_dir (str, optional): Root directory for saving the experiment. Defaults to "./outputs".
            tensorboard (bool, optional): Whether to use the tensorboard. Defaults to True.
            csv (bool, optional): Whether to save the result in csv format. Defaults to False.
            image (bool, optional): Whether to save the validation results. It is ignored if the task is classification. Defaults to True.
            stdout (bool, optional): Whether to log the standard output. Defaults to True.
            save_optimizer_state (bool, optional): Whether to save optimizer state with model checkpoint to resume training. Defaults to True.
            validation_epoch (int, optional): Validation frequency in total training process. Defaults to 10.
            save_checkpoint_epoch (int, optional): Checkpoint saving frequency in total training process. Defaults to None.
        """

        self.logging = LoggingConfig(
            project_id=project_id,
            output_dir=output_dir,
            tensorboard=tensorboard,
            csv=csv,
            image=image,
            stdout=stdout,
            save_optimizer_state=save_optimizer_state,
            validation_epoch=validation_epoch,
            save_checkpoint_epoch=save_checkpoint_epoch,
        )

    def set_environment_config(self, seed: int = 1, num_workers: int = 4):
        """Set the environment configuration.

        Args:
            seed (int, optional): Random seed. Defaults to 1.
            num_workers (int, optional): The number of multi-processing workers to be used by the data loader. Defaults to 4.
        """

        self.environment = EnvironmentConfig(seed=seed, num_workers=num_workers)

    def _change_transforms(self, transforms: Transform):
        """Update the 'size' attribute in the given list of transforms with the specified image size.

        Args:
            transforms (List[Transform]): The list of transforms to be updated.

        Returns:
            List[Transform]: The list of transforms with the 'size' attribute updated to the specified image size.
        """

        field_name_to_check = "size"

        if transforms is None:
            return transforms

        for transform in transforms:
            field_type = transform.__annotations__.get(field_name_to_check)

            if field_type == List:
                transform.size = [self.img_size, self.img_size]
            elif field_type == int:
                transform.size = self.img_size

        return transforms

    def _apply_img_size(self):
        """Apply the specified image size to the augmentation configurations.

        This method updates the 'img_size' attribute in the augmentation configurations, including
        'train.transforms', 'train.mix_transforms', and 'inference.transforms'.
        """

        self.augmentation.img_size = self.img_size
        self.augmentation.train.transforms = self._change_transforms(self.augmentation.train.transforms)
        self.augmentation.train.mix_transforms = self._change_transforms(self.augmentation.train.mix_transforms)
        self.augmentation.inference.transforms = self._change_transforms(self.augmentation.inference.transforms)

    def train(self, gpus: str, project_name: str) -> Dict:
        """Train the model with the specified configuration.

        Args:
            gpus (str): GPU ids to use, separated by commas.
            project_name (str): Project name to save the experiment.

        Returns:
            Dict: A dictionary containing information about the training.
        """

        self._validate_config()
        self._apply_img_size()
        self.logging.project_id = project_name

        destination_folder = Path(self.logging.output_dir) / self.logging.project_id
        FileHandler.create_unique_folder(folder_path=destination_folder)
        metadata = MetadataHandler.init_metadata(folder_path=destination_folder, task_type=TaskType.TRAIN)

        configs = TrainerConfigs(
            self.data,
            self.augmentation,
            self.model,
            self.training,
            self.logging,
            self.environment,
        )

        logging_dir = train_with_yaml(
            gpus=gpus,
            data=configs.data,
            augmentation=configs.augmentation,
            model=configs.model,
            training=configs.training,
            logging=configs.logging,
            environment=configs.environment,
        )
        training_summary_path = logging_dir / "training_summary.json"
        training_summary = FileHandler.load_json(file_path=training_summary_path)
        is_success = training_summary["success"]
        status = Status.COMPLETED if is_success else Status.STOPPED

        FileHandler.remove_folder(configs.temp_folder)
        logger.info(f"Removed {configs.temp_folder} folder.")

        destination_folder = Path(self.logging.output_dir) / self.logging.project_id
        FileHandler.move_and_cleanup_folders(source_folder=logging_dir, destination_folder=destination_folder)
        logger.info(f"Files in {logging_dir} were moved to {destination_folder}.")

        best_fx_paths = list(Path(destination_folder).glob("*best_fx.pt"))
        best_onnx_paths = list(Path(destination_folder).glob("*best.onnx"))
        hparams_path = destination_folder / "hparams.yaml"

        if best_fx_paths:
            metadata.update_best_fx_model_path(best_fx_model_path=best_fx_paths[0].as_posix())
        if best_onnx_paths:
            metadata.update_best_onnx_model_path(best_onnx_model_path=best_onnx_paths[0].as_posix())
        metadata.update_model_info(
            task=self.task,
            model=self.model.name,
            dataset=self.data.name,
            input_shapes=[InputShape(batch=1, channel=3, dimension=[self.img_size, self.img_size])],
        )
        metadata.update_training_info(epoch=self.training.epochs, batch_size=self.training.batch_size)
        metadata.update_training_result(training_summary=training_summary)
        metadata.update_logging_dir(logging_dir=destination_folder.as_posix())
        metadata.update_hparams(hparams=hparams_path.as_posix())
        metadata.update_status(status=status)
        MetadataHandler.save_json(data=metadata.asdict(), folder_path=destination_folder)

        return metadata.asdict()



from .action import ConvertAction, ExperimentAction
from .experiment import CheckpointChooseMethod, EncryptionKey, NetworkArch

__all__ = [
    "ExperimentAction",
    "ConvertAction",
    "NetworkArch",
    "CheckpointChooseMethod",
    "EncryptionKey",
]



from .augmentation import AUGMENTATION_CONFIG_TYPE
from .data import DATA_CONFIG_TYPE
from .model import CLASSIFICATION_MODELS, DETECTION_MODELS, SEGMENTATION_MODELS
from .training import TRAINING_CONFIG_TYPE

__all__ = [
    "DATA_CONFIG_TYPE",
    "CLASSIFICATION_MODELS",
    "DETECTION_MODELS",
    "SEGMENTATION_MODELS",
    "TRAINING_CONFIG_TYPE",
    "AUGMENTATION_CONFIG_TYPE",
]



from enum import Enum


class TaskType(str, Enum):
    TRAIN = "train"
    COMPRESS = "compress"
    CONVERT = "convert"
    BENCHMARK = "benchmark"


class Status(str, Enum):
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    STOPPED = "stopped"
    ERROR = "error"



from netspresso_trainer.cfg.model import (
    ClassificationEfficientFormerModelConfig,
    ClassificationMixNetLargeModelConfig,
    ClassificationMixNetMediumModelConfig,
    ClassificationMixNetSmallModelConfig,
    ClassificationMobileNetV3ModelConfig,
    ClassificationMobileViTModelConfig,
    ClassificationResNetModelConfig,
    ClassificationViTModelConfig,
    DetectionEfficientFormerModelConfig,
    DetectionMixNetLargeModelConfig,
    DetectionMixNetMediumModelConfig,
    DetectionMixNetSmallModelConfig,
    DetectionMobileNetV3ModelConfig,
    DetectionResNetModelConfig,
    DetectionYoloXModelConfig,
    PIDNetModelConfig,
    SegmentationEfficientFormerModelConfig,
    SegmentationMixNetLargeModelConfig,
    SegmentationMixNetMediumModelConfig,
    SegmentationMixNetSmallModelConfig,
    SegmentationMobileNetV3ModelConfig,
    SegmentationResNetModelConfig,
    SegmentationSegFormerModelConfig,
)

CLASSIFICATION_MODELS = {
    "EfficientFormer": ClassificationEfficientFormerModelConfig,
    "MobileNetV3": ClassificationMobileNetV3ModelConfig,
    "MobileViT": ClassificationMobileViTModelConfig,
    "ResNet": ClassificationResNetModelConfig,
    "ViT": ClassificationViTModelConfig,
    "MixNetS": ClassificationMixNetSmallModelConfig,
    "MixNetM": ClassificationMixNetMediumModelConfig,
    "MixNetL": ClassificationMixNetLargeModelConfig,
}

DETECTION_MODELS = {
    "EfficientFormer": DetectionEfficientFormerModelConfig,
    "YOLOX-S": DetectionYoloXModelConfig,
    "ResNet": DetectionResNetModelConfig,
    "MobileNetV3": DetectionMobileNetV3ModelConfig,
    "MixNetL": DetectionMixNetLargeModelConfig,
    "MixNetM": DetectionMixNetMediumModelConfig,
    "MixNetS": DetectionMixNetSmallModelConfig,
}

SEGMENTATION_MODELS = {
    "EfficientFormer": SegmentationEfficientFormerModelConfig,
    "MobileNetV3": SegmentationMobileNetV3ModelConfig,
    "ResNet": SegmentationResNetModelConfig,
    "SegFormer": SegmentationSegFormerModelConfig,
    "MixNetS": SegmentationMixNetSmallModelConfig,
    "MixNetM": SegmentationMixNetMediumModelConfig,
    "MixNetL": SegmentationMixNetLargeModelConfig,
    "PIDNet": PIDNetModelConfig,
}



from enum import Enum
from typing import Literal


class DeviceName(str, Enum):
    RASPBERRY_PI_5 = "RaspberryPi5"
    RASPBERRY_PI_4B = "RaspberryPi4B"
    RASPBERRY_PI_3B_PLUS = "RaspberryPi3BPlus"
    RASPBERRY_PI_ZERO_W = "RaspberryPi-ZeroW"
    RASPBERRY_PI_ZERO_2W = "RaspberryPi-Zero2W"
    RENESAS_RZ_V2L = "rzv2l_avnet"
    RENESAS_RZ_V2M = "rzv2m"
    JETSON_NANO = "Jetson-Nano"
    JETSON_TX2 = "Jetson-Tx2"
    JETSON_XAVIER = "Jetson-Xavier"
    JETSON_NX = "Jetson-Nx"
    JETSON_AGX_ORIN = "Jetson-AGX-Orin"
    JETSON_ORIN_NANO = "Jetson-Orin-Nano"
    AWS_T4 = "AWS-T4"
    INTEL_XEON_W_2233 = "Intel-Xeon"
    ALIF_ENSEMBLE_E7_DEVKIT_GEN2 = "Ensemble-E7-DevKit-Gen2"
    RENESAS_RA8D1 = "Renesas-RA8D1"
    ARM_ETHOS_U_SERIES = "Arm Virtual Hardware Ethos-U Series"

    @classmethod
    def create_literal(cls):
        return Literal[
            "RaspberryPi5",
            "RaspberryPi4B",
            "RaspberryPi3BPlus",
            "RaspberryPi-ZeroW",
            "RaspberryPi-Zero2W",
            "rzv2l_avnet",
            "rzv2m",
            "Jetson-Nano",
            "Jetson-Tx2",
            "Jetson-Xavier",
            "Jetson-Nx",
            "Jetson-AGX-Orin",
            "Jetson-Orin-Nano",
            "AWS-T4",
            "Intel-Xeon",
            "Ensemble-E7-DevKit-Gen2",
            "Renesas-RA8D1",
            "Arm Virtual Hardware Ethos-U Series",
        ]

    JETSON_DEVICES = [
        JETSON_NANO,
        JETSON_TX2,
        JETSON_XAVIER,
        JETSON_NX,
        JETSON_AGX_ORIN,
        JETSON_ORIN_NANO,
    ]
    RASPBERRY_PI_DEVICES = [
        RASPBERRY_PI_5,
        RASPBERRY_PI_4B,
        RASPBERRY_PI_3B_PLUS,
        RASPBERRY_PI_ZERO_W,
        RASPBERRY_PI_ZERO_2W,
    ]
    RENESAS_DEVICES = [RENESAS_RZ_V2L, RENESAS_RZ_V2M]
    NVIDIA_GRAPHIC_CARDS = [AWS_T4]
    INTEL_DEVICES = [INTEL_XEON_W_2233]
    AVAILABLE_INT8_DEVICES = [
        ALIF_ENSEMBLE_E7_DEVKIT_GEN2,
        RENESAS_RA8D1,
        RASPBERRY_PI_5,
        RASPBERRY_PI_4B,
        RASPBERRY_PI_3B_PLUS,
        RASPBERRY_PI_ZERO_W,
        RASPBERRY_PI_ZERO_2W,
        ARM_ETHOS_U_SERIES,
    ]
    ONLY_INT8_DEVICES = [ALIF_ENSEMBLE_E7_DEVKIT_GEN2, RENESAS_RA8D1, ARM_ETHOS_U_SERIES]


class SoftwareVersion(str, Enum):
    JETPACK_4_4_1 = "4.4.1-b50"
    JETPACK_4_6 = "4.6-b199"
    JETPACK_5_0_1 = "5.0.1-b118"
    JETPACK_5_0_2 = "5.0.2-b231"
    JETPACK_6_0 = "6.0-b52"

    @classmethod
    def create_literal(cls):
        return Literal["4.4.1-b50", "4.6-b199", "5.0.1-b118", "5.0.2-b231", "6.0-b52"]


class HardwareType(str, Enum):
    HELIUM = "helium"

    @classmethod
    def create_literal(cls):
        return Literal["helium"]


class TaskStatus(str, Enum):
    IN_QUEUE = "IN_QUEUE"
    IN_PROGRESS = "IN_PROGRESS"
    FINISHED = "FINISHED"
    ERROR = "ERROR"
    USER_CANCEL = "USER_CANCEL"

    @classmethod
    def create_literal(cls):
        return Literal["IN_QUEUE", "IN_PROGRESS", "FINISHED", "ERROR", "USER_CANCEL"]


device_name_literal = DeviceName.create_literal()



import json
from dataclasses import asdict, dataclass, field
from typing import Any, Dict, List

from netspresso.enums.metadata import Status, TaskType

from .common import TargetDevice


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class BenchmarkInfo:
    target_device: str = ""
    filename: str = ""
    data_type: str = ""
    processor: str = ""
    software_version: str = ""
    hardware_type: str = ""
    input_model_uuid: str = ""
    benchmark_task_uuid: str = ""
    devicefarm_benchmark_task_uuid: str = ""
    devicefarm_model_uuid: str = ""


@dataclass
class Result:
    memory_footprint_gpu: int = 0
    memory_footprint_cpu: int = 0
    latency: int = 0
    ram_size: int = 0
    power_consumption: int = 0
    file_size: int = 0


@dataclass
class BenchmarkerMetadata:
    status: Status = Status.IN_PROGRESS
    task_type: TaskType = TaskType.BENCHMARK
    benchmark_info: BenchmarkInfo = field(default_factory=BenchmarkInfo)
    result: Result = field(default_factory=Result)

    def asdict(self) -> Dict:
        _dict = json.loads(json.dumps(asdict(self)))
        return _dict

    def update_status(self, status: Status) -> None:
        self.status = status

    def update_benchmark_info(
        self,
        target_device: str,
        file_name: str,
        data_type: str,
        processor: str,
        software_version: str,
        hardware_type: str,
        input_model_uuid: str,
        benchmark_task_uuid: str,
        devicefarm_benchmark_task_uuid: str,
        devicefarm_model_uuid: str,
    ) -> None:
        self.benchmark_info.target_device = target_device
        self.benchmark_info.filename = file_name
        self.benchmark_info.data_type = data_type
        self.benchmark_info.processor = processor
        self.benchmark_info.software_version = software_version
        self.benchmark_info.hardware_type = hardware_type
        self.benchmark_info.input_model_uuid = input_model_uuid
        self.benchmark_info.benchmark_task_uuid = benchmark_task_uuid
        self.benchmark_info.devicefarm_benchmark_task_uuid = devicefarm_benchmark_task_uuid
        self.benchmark_info.devicefarm_model_uuid = devicefarm_model_uuid

    def update_result(
        self,
        memory_footprint_gpu: int,
        memory_footprint_cpu: int,
        latency: int,
        ram_size: int,
        power_consumption: int,
        file_size: int,
    ) -> None:
        self.result.memory_footprint_gpu = memory_footprint_gpu
        self.result.memory_footprint_cpu = memory_footprint_cpu
        self.result.latency = latency
        self.result.ram_size = ram_size
        self.result.power_consumption = power_consumption
        self.result.file_size = file_size



from typing import Optional, Union

from netspresso.benchmarker import Benchmarker
from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.schemas import UserInfo
from netspresso.clients.tao import TAOTokenHandler
from netspresso.compressor import Compressor
from netspresso.converter import Converter
from netspresso.enums import Task
from netspresso.tao import TAOTrainer
from netspresso.trainer import Trainer


class NetsPresso:
    def __init__(self, email: str, password: str, verify_ssl: bool = True) -> None:
        """Initialize NetsPresso instance and perform user authentication.

        Args:
            email (str): User's email for authentication.
            password (str): User's password for authentication.
            verify_ssl (bool): Flag to indicate whether SSL certificates should be verified. Defaults to True.
        """
        self.token_handler = TokenHandler(email=email, password=password, verify_ssl=verify_ssl)
        self.user_info = self.get_user()

    def get_user(self) -> UserInfo:
        """Get user information using the access token.

        Returns:
            UserInfo: User information.
        """
        user_info = auth_client.get_user_info(self.token_handler.tokens.access_token, self.token_handler.verify_ssl)
        return user_info

    def trainer(self, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None) -> Trainer:
        """Initialize and return a Trainer instance.

        Args:
            task (Union[str, Task], optional): Type of task (classification, detection, segmentation).
            yaml_path (str, optional): Path to the YAML configuration file.

        Returns:
            Trainer: Initialized Trainer instance.
        """
        return Trainer(task=task, yaml_path=yaml_path)

    def compressor(self) -> Compressor:
        """Initialize and return a Compressor instance.

        Returns:
            Compressor: Initialized Compressor instance.
        """
        return Compressor(token_handler=self.token_handler)

    def converter(self) -> Converter:
        """Initialize and return a Converter instance.

        Returns:
            Converter: Initialized Converter instance.
        """
        return Converter(token_handler=self.token_handler, user_info=self.user_info)

    def benchmarker(self) -> Benchmarker:
        """Initialize and return a Benchmarker instance.

        Returns:
            Benchmarker: Initialized Benchmarker instance.
        """
        return Benchmarker(token_handler=self.token_handler, user_info=self.user_info)


class TAO:
    def __init__(self, ngc_api_key: str) -> None:
        """Initialize TAO instance and perform user authentication.

        Args:
            ngc_api_key (str): API key for TAO authentication.
        """
        self.ngc_api_key = ngc_api_key
        self.token_handler = TAOTokenHandler(ngc_api_key=ngc_api_key)

    def trainer(self) -> TAOTrainer:
        """Initialize and return a Trainer instance.

        Returns:
            TAO: Initialized Trainer instance.
        """
        return TAOTrainer(token_handler=self.token_handler)



from dataclasses import dataclass


@dataclass
class TargetDevice:
    display_name: str = ""
    display_brand_name: str = ""
    device_name: str = ""
    software_version: str = ""
    software_version_display_name: str = ""
    hardware_type: str = ""



from .model import (
    BaseRequestModel,
    BaseTaskModel,
    BenchmarkTask,
    ConversionTask,
    InputShape,
    Model,
    ModelBenchmarkRequest,
    ModelConversionRequest,
    TargetDevice,
    TargetDeviceFilter,
)

__all__ = [
    "InputShape",
    "TargetDevice",
    "Model",
    "BaseRequestModel",
    "ModelBenchmarkRequest",
    "ModelConversionRequest",
    "BaseTaskModel",
    "BenchmarkTask",
    "ConversionTask",
    "TargetDeviceFilter",
]



import configparser
import os
from enum import Enum
from pathlib import Path

from loguru import logger

BASE_DIR = Path(__file__).resolve().parent
config_parser = configparser.ConfigParser()
DEPLOYMENT_MODE = os.getenv("DEPLOYMENT_MODE", "PROD")
logger.info(f"Read {DEPLOYMENT_MODE} config")
config_parser.read(f"{BASE_DIR}/configs/config-{DEPLOYMENT_MODE.lower()}.ini")


class EnvironmentType(str, Enum):
    PROD = "prod"
    LOCAL = "local"


class Module(str, Enum):
    GENERAL = "GENERAL"
    COMPRESSOR = "COMPRESSOR"
    LAUNCHER = "LAUNCHER"
    TAO = "TAO"


class EndPointProperty(str, Enum):
    HOST = "HOST"
    PORT = "PORT"
    URI_PREFIX = "URI_PREFIX"


class Config:
    def __init__(self, module: Module = Module.GENERAL):
        self.ENVIRONMENT_TYPE = EnvironmentType(DEPLOYMENT_MODE.lower())
        self.MODULE = module
        self.HOST = config_parser[self.MODULE][EndPointProperty.HOST]
        self.PORT = int(config_parser[self.MODULE][EndPointProperty.PORT])
        self.URI_PREFIX = config_parser[self.MODULE][EndPointProperty.URI_PREFIX]



import json
from datetime import datetime

import jwt
import pytz
import requests
from loguru import logger

from netspresso.clients.auth.schemas import LoginRequest, LoginResponse, Tokens, UserInfo
from netspresso.clients.config import Config, Module
from netspresso.clients.utils import get_headers


class AuthClient:
    def __init__(self, config: Config = Module.GENERAL):
        """Initialize the UserSession.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
        """

        self.config = Config(config)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.uri_prefix = self.config.URI_PREFIX
        self.base_url = f"{self.host}:{self.port}{self.uri_prefix}"

    def login(self, email, password, verify_ssl: bool = True) -> Tokens:
        try:
            url = f"{self.base_url}/auth/local/login"
            data = LoginRequest(username=email, password=password)
            response = requests.post(url, json=data.dict(), headers=get_headers(), verify=verify_ssl)
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                session = LoginResponse(**response_body)
                logger.info("Login successfully")
                return session.tokens
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Login failed. Error: {e}")
            raise e

    def get_user_info(self, access_token, verify_ssl: bool = True) -> UserInfo:
        try:
            url = f"{self.base_url}/user"
            response = requests.get(url, headers=get_headers(access_token=access_token), verify=verify_ssl)
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                user_info = UserInfo(**response_body)
                logger.info("Successfully got user information")
                return user_info
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Failed to get user information. Error: {e}")
            raise e

    def get_credit(self, access_token, verify_ssl: bool = True) -> int:
        user_info = self.get_user_info(access_token, verify_ssl)

        return user_info.total

    def reissue_token(self, access_token, refresh_token, verify_ssl: bool = True) -> Tokens:
        try:
            url = f"{self.base_url}/auth/token"
            data = Tokens(access_token=access_token, refresh_token=refresh_token)
            response = requests.post(url, data=data.json(), headers=get_headers(json_type=True), verify=verify_ssl)
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                tokens = Tokens(**response_body["tokens"])
                logger.info("Successfully reissued token")
                return tokens
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            raise e


class TokenHandler:
    def __init__(self, email, password, verify_ssl: bool = True) -> None:
        self.tokens = auth_client.login(email=email, password=password, verify_ssl=verify_ssl)
        self.email = email
        self.password = password
        self.verify_ssl = verify_ssl

    def check_jwt_exp(self):
        payload = jwt.decode(self.tokens.access_token, options={"verify_signature": False})
        return datetime.now(pytz.utc).timestamp() <= payload["exp"]

    def validate_token(self):
        if not self.check_jwt_exp():
            try:
                self.tokens = auth_client.reissue_token(
                    self.tokens.access_token, self.tokens.refresh_token, self.verify_ssl
                )
            except Exception:
                self.tokens = auth_client.login(email=self.email, password=self.password, verify_ssl=self.verify_ssl)
                logger.info("The refresh token has expired. the token has been reissued.")


auth_client = AuthClient()



import tempfile
from dataclasses import is_dataclass
from pathlib import Path
from typing import Union

from netspresso_trainer.cfg import (
    AugmentationConfig,
    DatasetConfig,
    EnvironmentConfig,
    LoggingConfig,
    ModelConfig,
    ScheduleConfig,
)
from omegaconf import OmegaConf


class TrainerConfigs:
    def __init__(
        self,
        data: Union[DatasetConfig, Path],
        augmentation: Union[AugmentationConfig, Path],
        model: Union[ModelConfig, Path],
        training: Union[ScheduleConfig, Path],
        logging: Union[LoggingConfig, Path],
        environment: Union[EnvironmentConfig, Path],
        prefix: str = "temp_np_trainer_configs_",
    ) -> None:
        """Initialize TrainerConfigs with specified configurations.

        Args:
            data (Union[DatasetConfig, Path]): Dataset configuration or path to a YAML file.
            augmentation (Union[AugmentationConfig, Path]): Augmentation configuration or path to a YAML file.
            model (Union[ModelConfig, Path]): Model configuration or path to a YAML file.
            training (Union[ScheduleConfig, Path]): Training schedule configuration or path to a YAML file.
            logging (Union[LoggingConfig, Path]): Logging configuration or path to a YAML file.
            environment (Union[EnvironmentConfig, Path]): Environment configuration or path to a YAML file.
            prefix (str, optional): Prefix for the temporary folder. Defaults to "temp_np_trainer_configs_".
        """

        self.prefix = prefix
        self.temp_folder = self.create_temp_folder()
        self.data = self.handle_config(data, "data")
        self.augmentation = self.handle_config(augmentation, "augmentation")
        self.model = self.handle_config(model, "model")
        self.training = self.handle_config(training, "training")
        self.logging = self.handle_config(logging, "logging")
        self.environment = self.handle_config(environment, "environment")

    def create_temp_folder(self) -> str:
        """Create a temporary folder and return its path.

        Returns:
            str: Path to the created temporary folder.
        """

        return tempfile.mkdtemp(prefix=self.prefix)

    def _save_config_as_yaml(self, config, type: str) -> Path:
        """Save the given configuration as a YAML file in the temporary folder.

        Args:
            config: Configuration to be saved.
            type (str): Type of the configuration.

        Returns:
            Path: Path to the saved YAML file.
        """

        config = OmegaConf.create({type: config})
        yaml_path = Path(self.temp_folder) / f"{type}.yaml"
        OmegaConf.save(config=config, f=yaml_path)

        return yaml_path

    def handle_config(self, config, type: str) -> Path:
        """Handle the input configuration, either saving it as a YAML file or using YAML file path directly.

        Args:
            config: Input configuration.
            type (str): Type of the configuration.

        Returns:
            Path: Path to the saved YAML file if the input is a dataclass; otherwise, the YAML file path itself.
        """

        if is_dataclass(config):
            return self._save_config_as_yaml(config, type)
        else:
            return config



from netspresso_trainer.cfg.augmentation import (
    ColorJitter,
    Pad,
    RandomCrop,
    RandomCutmix,
    RandomHorizontalFlip,
    RandomMixup,
    RandomResizedCrop,
    RandomVerticalFlip,
    Resize,
    TrivialAugmentWide,
)

__all__ = [
    "ColorJitter",
    "Pad",
    "RandomCrop",
    "RandomResizedCrop",
    "RandomHorizontalFlip",
    "RandomVerticalFlip",
    "Resize",
    "TrivialAugmentWide",
    "RandomMixup",
    "RandomCutmix",
]



from pydantic import BaseModel, EmailStr, Field


class LoginRequest(BaseModel):
    username: str = Field(..., description="User Name")
    password: str = Field(..., description="Password")


class Tokens(BaseModel):
    access_token: str = Field(..., description="Access Token")
    refresh_token: str = Field(..., description="Refresh Token")


class LoginResponse(BaseModel):
    current_time: str = Field(..., description="Login Time")
    region: str = Field(..., description="User Resion")
    tokens: Tokens = Field(default_factory=Tokens, description="Session Token")


class CreditResponse(BaseModel):
    free: int = Field(..., description="Free Credit")
    reward: int = Field(..., description="Reward Credit")
    contract: int = Field(..., description="Contract Credit")
    paid: int = Field(..., description="Paid Credit")
    total: int = Field(..., description="Total Credit")


class UserDetailResponse(BaseModel):
    first_name: str = Field(..., description="First Name")
    last_name: str = Field(..., description="Last Name")
    company: str = Field(..., description="Company")


class UserInfo(BaseModel):
    user_id: str = Field(..., description="User ID")
    email: EmailStr = Field(..., description="Email")
    username: str = Field(..., description="User Name")
    detail_data: UserDetailResponse = Field(..., description="User Detail")
    is_active: bool = Field(..., description="Active Status")
    is_admin: bool = Field(..., description="Admin Status")
    current_time: str = Field(..., description="Current Time")
    created_time: str = Field(..., description="Created Time")
    last_login_time: str = Field(..., description="Last Login Time")
    free: int = Field(0, description="Free Credit")
    reward: int = Field(0, description="Reward Credit")
    contract: int = Field(0, description="Contract Credit")
    paid: int = Field(0, description="Paid Credit")
    total: int = Field(0, description="Total Credit")



import json
from pathlib import Path
from typing import Any, Dict

from netspresso.enums import TaskType

from .default import BenchmarkerMetadata, CompressorMetadata, ConverterMetadata, TrainerMetadata


class MetadataHandler:
    @staticmethod
    def save_json(data: dict, folder_path: str, file_name: str = "metadata") -> None:
        """Save dictionary data to a JSON file.

        Args:
            data (dict): The dictionary data to be saved.
            file_path (str): The path to the JSON file.

        Returns:
            None
        """
        file_path = Path(folder_path) / f"{file_name}.json"

        with open(file_path, "w") as json_file:
            json.dump(data, json_file, indent=4)

    @staticmethod
    def load_json(file_path: str) -> Dict[str, Any]:
        """Load JSON data from a file.

        Args:
            file_path (str): The path to the JSON file.

        Returns:
            dict: Loaded dictionary data.
        """
        with open(file_path, "r") as json_file:
            data = json.load(json_file)
        return data

    @staticmethod
    def get_default_metadata(task_type: TaskType) -> Any:
        """Get the default metadata object based on the task type.

        Args:
            task_type (TaskType): The type of the task.

        Returns:
            Any: Default metadata object.
        """
        if task_type == TaskType.TRAIN:
            _metadata = TrainerMetadata()
        elif task_type == TaskType.COMPRESS:
            _metadata = CompressorMetadata()
        elif task_type == TaskType.CONVERT:
            _metadata = ConverterMetadata()
        elif task_type == TaskType.BENCHMARK:
            _metadata = BenchmarkerMetadata()
        return _metadata

    @staticmethod
    def init_metadata(folder_path: str, task_type: TaskType, file_name: str = "metadata") -> Any:
        """Initialize metadata by saving default metadata to a JSON file.

        Args:
            folder_path (str): The path to the folder where metadata will be stored.
            task_type (TaskType): The type of the task.

        Returns:
            Any: Default metadata object.
        """
        default_metadata = MetadataHandler.get_default_metadata(task_type)
        MetadataHandler.save_json(default_metadata.asdict(), folder_path, file_name=file_name)

        return default_metadata



from netspresso.enums.compression import CompressionMethod


class CompressionParamsValidator:
    def __init__(self, compression_method, layers):
        self.compression_method = compression_method
        self.layers = layers
        self.supported_method = list(CompressionMethod.__members__.keys())

    def validate(self):
        compression_methods = {
            "PR_L2": self._validate_pr_ratio,
            "PR_GM": self._validate_pr_ratio,
            "PR_NN": self._validate_pr_ratio,
            "PR_ID": self._validate_pr_index,
            "FD_TK": self._validate_fd_rank2,
            "FD_SVD": self._validate_fd_rank1,
            "FD_CP": self._validate_fd_rank1,
        }

        validation_method = compression_methods.get(self.compression_method)
        if validation_method:
            validation_method()
        else:
            raise ValueError(
                f"Invalid compression_method: {self.compression_method}. Please choose from {self.supported_method}."
            )

    def _validate_pr_ratio(self):
        for layer in self.layers:
            if not layer.use:
                continue
            if len(layer.values) != 1:
                raise ValueError(f"The number of values should be 1, but got {len(layer.values)}.")
            if not isinstance(layer.values[0], float):
                raise ValueError(f"The type of the input value should be float, but got {layer.values}.")
            if not 0.0 < layer.values[0] <= 1.0:
                raise ValueError(f"The range of input value should be 0.0 < x <= 1.0, but got {layer.values}.")

    def _validate_pr_index(self):
        for layer in self.layers:
            if not layer.use:
                continue
            if layer.channels[0] <= len(layer.values):
                raise ValueError(
                    f"The number of values should be less than {layer.channels[0]}, but got {len(layer.values)}."
                )
            if not all(isinstance(x, int) for x in layer.values):
                raise ValueError(f"The type of the input values should be integer, but got {layer.values}")
            if not all(0 <= x < layer.channels[0] for x in layer.values):
                raise ValueError(
                    f"The range of input values should be 0 <= x < out channels({layer.channels[0]}), but got {layer.values}"
                )

    def _validate_fd_rank2(self):
        for layer in self.layers:
            if not layer.use:
                continue
            if len(layer.values) != 2:
                raise ValueError(f"The number of values should be 2, but got {len(layer.values)}.")
            if not all(isinstance(x, int) for x in layer.values):
                raise ValueError(f"The type of the input values should be integer, but got {layer.values}")
            if not all(x > 0 and x <= channel for x, channel in zip(layer.values, layer.channels)):
                raise ValueError(
                    f"The range of input values should be 0 < x <= channels({layer.channels}), but got {layer.values}"
                )

    def _validate_fd_rank1(self):
        for layer in self.layers:
            if not layer.use:
                continue
            if len(layer.values) != 1:
                raise ValueError(f"The number of values should be 1, but got {len(layer.values)}.")
            if not isinstance(layer.values[0], int):
                raise ValueError(f"The type of the input values should be integer, but got {layer.values}")
            if not 0 < layer.values[0] <= min(layer.channels):
                raise ValueError(
                    f"The range of input values should be 0 < x < min(in channels, out channels)({min(layer.channels)}), but got {layer.values}"
                )



import time
from pathlib import Path
from typing import Dict, Optional, Union

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.schemas.auth import UserInfo
from netspresso.clients.launcher import launcher_client
from netspresso.clients.launcher.schemas import BenchmarkTask, TargetDeviceFilter
from netspresso.enums.credit import ServiceCredit
from netspresso.enums.device import DeviceName, HardwareType, SoftwareVersion, TaskStatus
from netspresso.enums.metadata import Status, TaskType
from netspresso.enums.model import DataType
from netspresso.enums.module import Module

from ..utils import FileHandler, check_credit_balance
from ..utils.metadata import MetadataHandler


class Benchmarker:
    def __init__(self, token_handler: TokenHandler, user_info: UserInfo) -> None:
        """Initialize the Benchmarker."""

        self.token_handler = token_handler
        self.user_info = user_info

    def benchmark_model(
        self,
        input_model_path: str,
        target_device_name: DeviceName,
        target_data_type: DataType = DataType.FP16,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        target_hardware_type: Optional[Union[str, HardwareType]] = None,
        wait_until_done: bool = True,
    ) -> Dict:
        """Benchmark the specified model on the specified device.

        Args:
            input_model_path (str): The file path where the model is located.
            target_device_name (DeviceName): Target device name.
            target_data_type (DataType): Data type of the model.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version. Required if target_device_name is one of the Jetson devices.
            target_hardware_type (Union[str, HardwareType], optional): Hardware type. Acceleration options for processing the model inference.
            wait_until_done (bool): If True, wait for the conversion result before returning the function.
                                If False, request the conversion and return the function immediately.

        Raises:
            e: If an error occurs during the benchmarking of the model.

        Returns:
            Dict: Model benchmark task dictionary.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        try:
            folder_path = Path(input_model_path).parent

            metadata = MetadataHandler.get_default_metadata(TaskType.BENCHMARK)
            if FileHandler.check_exists(folder_path / "benchmark.json"):
                metadatas = MetadataHandler.load_json(folder_path / "benchmark.json")
                metadatas.append(metadata.asdict())
            else:
                metadatas = [metadata.asdict()]
            MetadataHandler.save_json(metadatas, folder_path, file_name="benchmark")

            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, self.token_handler.verify_ssl
            )
            check_credit_balance(user_credit=current_credit, service_credit=ServiceCredit.MODEL_BENCHMARK)
            model = launcher_client.upload_model(
                model_file_path=input_model_path,
                target_function=Module.BENCHMARK,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            model_uuid = model.model_uuid

            if target_device_name in DeviceName.JETSON_DEVICES and target_software_version is None:
                raise ValueError(
                    "The benchmark is unavailable. Please set JetPack version with target_software_version for Jetson Devices."
                )

            # Check available int8 converting devices
            if target_data_type == DataType.INT8:
                if target_device_name not in DeviceName.AVAILABLE_INT8_DEVICES:
                    raise ValueError(f"int8 converting supports only {DeviceName.AVAILABLE_INT8_DEVICES}.")
            else:  # FP16, FP32
                if target_device_name in DeviceName.ONLY_INT8_DEVICES:
                    raise ValueError(f"{DeviceName.ONLY_INT8_DEVICES} only support int8 data types.")

            if target_hardware_type == HardwareType.HELIUM and target_device_name not in DeviceName.ONLY_INT8_DEVICES:
                raise ValueError(f"{DeviceName.ONLY_INT8_DEVICES} only support helium hardware type.")

            devices = TargetDeviceFilter.filter_devices_with_device_name(
                name=target_device_name, devices=model.available_devices
            )
            if target_device_name in DeviceName.JETSON_DEVICES:
                devices = TargetDeviceFilter.filter_devices_with_device_software_version(
                    software_version=target_software_version, devices=devices
                )
            devices = TargetDeviceFilter.filter_devices_with_hardware_type(
                hardware_type=target_hardware_type, devices=devices
            )

            if not devices:
                raise NotImplementedError(
                    "The benchmark is unavailable. There is no available device with given target_device_name and target_software_version."
                )

            target_device = devices[0]
            target_device_name = target_device.device_name
            target_software_version = target_device.software_version
            target_hardware_type = target_device.hardware_type

            if target_device_name is None:
                raise NotImplementedError(
                    "There is no avaliable function for given paremeter. Please specify the target device."
                )

            model_benchmark: BenchmarkTask = launcher_client.benchmark_model(
                user_uuid=self.user_info.user_id,
                model_uuid=model_uuid,
                target_device=target_device_name,
                data_type=target_data_type,
                software_version=target_software_version,
                hardware_type=target_hardware_type,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            model_benchmark = self.get_benchmark_task(benchmark_task=model_benchmark)

            if wait_until_done:
                while model_benchmark.status in [
                    TaskStatus.IN_QUEUE,
                    TaskStatus.IN_PROGRESS,
                ]:
                    model_benchmark = self.get_benchmark_task(benchmark_task=model_benchmark)
                    time.sleep(1)

            metadata.update_benchmark_info(
                target_device=model_benchmark.target_device,
                file_name=model_benchmark.filename,
                data_type=model_benchmark.data_type,
                processor=model_benchmark.processor,
                software_version=model_benchmark.software_version,
                hardware_type=model_benchmark.hardware_type,
                input_model_uuid=model_benchmark.input_model_uuid,
                benchmark_task_uuid=model_benchmark.benchmark_task_uuid,
                devicefarm_benchmark_task_uuid=model_benchmark.devicefarm_benchmark_task_uuid,
                devicefarm_model_uuid=model_benchmark.devicefarm_model_uuid,
            )
            metadata.update_result(
                memory_footprint_gpu=model_benchmark.memory_footprint_gpu,
                memory_footprint_cpu=model_benchmark.memory_footprint_cpu,
                latency=model_benchmark.latency,
                ram_size=model_benchmark.ram_size,
                power_consumption=model_benchmark.power_consumption,
                file_size=model_benchmark.file_size,
            )
            metadata.update_status(status=Status.COMPLETED)
            metadatas[-1] = metadata.asdict()
            MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name="benchmark")

            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.MODEL_BENCHMARK} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

        except Exception as e:
            logger.error(f"Benchmark failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            metadatas[-1] = metadata.asdict()
            MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name="benchmark")
            raise e

        except KeyboardInterrupt:
            metadata.update_status(status=Status.STOPPED)
            metadatas[-1] = metadata.asdict()
            MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name="benchmark")

        return metadata.asdict()

    def get_benchmark_task(self, benchmark_task: Union[str, BenchmarkTask]) -> BenchmarkTask:
        """Get information about the specified benchmark task using either the benchmark task object or its UUID.

        Args:
            benchmark_task (Union[BenchmarkTask, str]): Benchmark task object or the UUID of the benchmark task.

        Raises:
            e: If an error occurs while retrieving information about the benchmark task.

        Returns:
            BenchmarkTask: Model benchmark task object.
        """

        self.token_handler.validate_token()

        try:
            task_uuid = None
            if isinstance(benchmark_task, str):
                task_uuid = benchmark_task
            elif type(benchmark_task) is BenchmarkTask:
                task_uuid = benchmark_task.benchmark_task_uuid
            else:
                raise NotImplementedError(
                    "There is no available function for the given parameter. The 'benchmark_task' should be a UUID string or a ModelBenchmark object."
                )

            return launcher_client.get_benchmark(
                benchmark_task_uuid=task_uuid,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

        except Exception as e:
            logger.error(f"Get benchmark failed. Error: {e}")



from enum import Enum
from typing import Literal


class Framework(str, Enum):
    TENSORFLOW_KERAS = "tensorflow_keras"
    TENSORFLOW = "saved_model"
    PYTORCH = "pytorch"
    ONNX = "onnx"
    TENSORRT = "tensorrt"
    OPENVINO = "openvino"
    TENSORFLOW_LITE = "tensorflow_lite"
    DRPAI = "drpai"

    @classmethod
    def create_compressor_literal(cls):
        return Literal["tensorflow_keras", "pytorch", "onnx"]

    @classmethod
    def create_launcher_literal(cls):
        return Literal[
            "onnx",
            "tensorrt",
            "openvino",
            "tensorflow_lite",
            "drpai",
            "keras",
            "saved_model",
        ]


class Extension(str, Enum):
    H5 = "h5"
    ZIP = "zip"
    PT = "pt"
    ONNX = "onnx"

    @classmethod
    def create_literal(cls):
        return Literal["h5", "zip", "pt", "onnx"]


class OriginFrom(str, Enum):
    CUSTOM = "custom"
    NPMS = "npms"

    @classmethod
    def create_literal(cls):
        return Literal["custom", "npms"]


class DataType(str, Enum):
    FP32 = "FP32"
    FP16 = "FP16"
    INT8 = "INT8"
    NONE = ""

    @classmethod
    def create_literal(cls):
        return Literal["FP32", "FP16", "INT8", ""]


compressor_framework_literal = Framework.create_compressor_literal()
launcher_framework_literal = Framework.create_launcher_literal()
extension_literal = Extension.create_literal()
originfrom_literal = OriginFrom.create_literal()
datatype_literal = DataType.create_literal()



from typing import Any, List

from pydantic import BaseModel, Field, root_validator, validator

from netspresso.clients.compressor.utils.validator import CompressionParamsValidator
from netspresso.enums.compression import (
    GroupPolicy,
    LayerNorm,
    Policy,
    StepOp,
    compression_literal,
    grouppolicy_literal,
    layernorm_literal,
    policy_literal,
    recommendation_literal,
    stepop_literal,
)


class OptionsBase(BaseModel):
    reshape_channel_axis: int = Field(-1, description="Reshape Channel Axis")

    @validator("reshape_channel_axis")
    def validate_reshape_channel_axis(cls, value):
        valid_values = [0, 1, -1, -2]
        if value not in valid_values:
            raise ValueError(f"The reshape_channel_axis value is in the range [0, 1, -1, -2], but got {value}")
        return value


class Options(OptionsBase):
    policy: policy_literal = Field(Policy.AVERAGE, description="Policy")
    layer_norm: layernorm_literal = Field(LayerNorm.STANDARD_SCORE, description="layer Norm")
    group_policy: grouppolicy_literal = Field(GroupPolicy.AVERAGE, description="Group Policy")
    step_size: int = Field(2, description="Step Size")
    step_op: stepop_literal = Field(StepOp.ROUND, description="Step Operator")
    reverse: bool = Field(False, description="Reverse")


class CreateCompressionRequest(BaseModel):
    model_id: str = Field(..., description="Model ID")
    model_name: str = Field(..., description="Model Name")
    description: str = Field("", description="Description")
    compression_method: compression_literal = Field(..., description="Compression Method")
    options: Options = Field(default_factory=Options, description="Compression Options")


class AvailableLayerBase(BaseModel):
    name: str = Field(..., description="Layer Name")
    values: List[Any] = Field([], description="Compression Parameters")


class AvailableLayer(AvailableLayerBase):
    use: bool = Field(False, description="Compression Selection Status")
    channels: List[int] = Field([], description="Channel Info")


class CompressionResponse(BaseModel):
    new_model_id: str = Field(..., description="Model ID")
    compression_id: str = Field(..., description="Compression ID")
    compression_method: compression_literal = Field(..., description="Compression Method")
    available_layers: List[AvailableLayer] = Field([], description="Compressible Layers")


class RecommendationRequest(BaseModel):
    model_id: str = Field(..., description="Model ID")
    compression_id: str = Field(..., description="Compression ID")
    recommendation_method: recommendation_literal = Field(..., description="Recommendation Method")
    recommendation_ratio: float = Field(..., description="Recommendation Ratio")
    options: Options = Field(default_factory=Options, description="Compression Options")

    @root_validator
    def validate_ratio(cls, values):
        method = values.get("recommendation_method")
        ratio = values.get("recommendation_ratio")

        if method in ["slamp"]:
            assert 0 < ratio <= 1, "The ratio range for SLAMP is 0 < ratio < = 1."
        elif method in ["vbmf"]:
            assert -1 <= ratio <= 1, "The ratio range for VBMF is -1 <= ratio <= 1."
        return values


class RecommendationInfo(BaseModel):
    name: str = Field(..., description="Layer Name")
    values: List[Any] = Field(..., description="Recommended Values")


class RecommendationResponse(BaseModel):
    recommended_layers: List[RecommendationInfo] = Field([], description="Recommended Layers")


class CompressionRequest(BaseModel):
    compression_id: str = Field(..., description="Compression ID")
    compression_method: compression_literal = Field(..., description="Compression Method")
    layers: List[AvailableLayer] = Field([], description="Compressible Layers")
    options: Options = Field(default_factory=Options, description="Compression Options")
    compressed_model_id: str = Field(..., description="Compressed Model ID")

    @root_validator
    def validate_compression_params(cls, values):
        compression_method = values.get("compression_method")
        layers = values.get("layers")

        compression_params_validator = CompressionParamsValidator(compression_method, layers)
        compression_params_validator.validate()

        return values


class AutoCompressionRequest(BaseModel):
    model_id: str = Field(..., description="Model ID")
    model_name: str = Field(..., description="Model Name")
    description: str = Field("", description="Description")
    recommendation_ratio: float = Field(..., gt=0, le=1, description="Recommendation Ratio")
    save_path: str = Field(..., description="Compressed Model Save Path")


class UploadDatasetRequest(BaseModel):
    model_id: str = Field(..., description="Model ID")
    file_path: str = Field(..., description="Model Path")


class GetAvailableLayersRequest(BaseModel):
    model_id: str = Field(..., description="Model ID")
    compression_method: compression_literal = Field(..., description="Compression Method")
    options: Options = Field(default_factory=Options, description="Compression Options")


class GetAvailableLayersReponse(BaseModel):
    compression_method: compression_literal = Field(..., description="Compression Method")
    available_layers: List[AvailableLayer] = Field([], description="Compressible Layers")



from enum import Enum
from typing import Literal


class Module(str, Enum):
    CONVERT = "CONVERT"
    BENCHMARK = "BENCHMARK"

    @classmethod
    def create_literal(cls):
        return Literal["CONVERT", "BENCHMARK"]



from netspresso_trainer.cfg import (
    ClassificationAugmentationConfig,
    DetectionAugmentationConfig,
    SegmentationAugmentationConfig,
)

AUGMENTATION_CONFIG_TYPE = {
    "classification": ClassificationAugmentationConfig,
    "detection": DetectionAugmentationConfig,
    "segmentation": SegmentationAugmentationConfig,
}



from netspresso_trainer.cfg import (
    LocalClassificationDatasetConfig,
    LocalDetectionDatasetConfig,
    LocalSegmentationDatasetConfig,
)

DATA_CONFIG_TYPE = {
    "classification": LocalClassificationDatasetConfig,
    "detection": LocalDetectionDatasetConfig,
    "segmentation": LocalSegmentationDatasetConfig,
}



from enum import Enum


class EncryptionKey(str, Enum):
    NVIDIA_TAO = "nvidia_tao"
    NVIDIA_TLT = "nvidia_tlt"
    TLT_ENCODE = "tlt_encode"


class CheckpointChooseMethod(str, Enum):
    LATEST_MODEL = "latest_model"
    BEST_MODEL = "best_model"
    FROM_EPOCH_NUMBER = "from_epoch_number"


class NetworkArch(str, Enum):
    CLASSIFICATION_TF1 = "classification_tf1"
    CLASSIFICATION_TF2 = "classification_tf2"
    CLASSIFICATION_PYT = "classification_pyt"
    DETECTNET_V2 = "detectnet_v2"
    FASTER_RCNN = "faster_rcnn"
    YOLO_V3 = "yolo_v3"
    YOLO_V4 = "yolo_v4"
    YOLO_V4_TINY = "yolo_v4_tiny"
    SSD = "ssd"
    DSSD = "dssd"
    RETINANET = "retinanet"
    DEFORMABLE_DETR = "deformable_detr"
    DINO = "dino"
    EFFICIENTDET_TF1 = "efficientdet_tf1"
    EFFICIENTDET_TF2 = "efficientdet_tf2"
    OCDNET = "ocdnet"
    MASK_RCNN = "mask_rcnn"
    UNET = "unet"
    SEGFORMER = "segformer"
    MULTITASK_CLASSIFICATION = "multitask_classification"
    LPRNET = "lprnet"
    BPNET = "bpnet"
    FPENET = "fpenet"
    ACTION_RECOGNITION = "action_recognition"
    MAL = "mal"
    ML_RECOG = "ml_recog"
    OCRNET = "ocrnet"
    OPTICAL_INSPECTION = "optical_inspection"
    POINTPILLARS = "pointpillars"
    POSE_CLASSIFICATION = "pose_classification"
    RE_IDENTIFICATION = "re_identification"
    VISUAL_CHANGENET = "visual_changenet"
    CENTERPOSE = "centerpose"
    ANNOTATIONS = "annotations"
    ANALYTICS = "analytics"
    AUGMENTATION = "augmentation"
    AUTO_LABEL = "auto_label"



from netspresso.clients.compressor.schemas.compression import Options

from .compression import CompressionMethod, GroupPolicy, LayerNorm, Policy, RecommendationMethod, StepOp
from .credit import ServiceCredit
from .device import DeviceName, HardwareType, SoftwareVersion, TaskStatus
from .metadata import Status, TaskType
from .model import DataType, Extension, Framework, OriginFrom
from .module import Module
from .tao.action import ConvertAction, ExperimentAction
from .task import Task

__all__ = [
    "ServiceCredit",
    "TaskType",
    "Status",
    "CompressionMethod",
    "RecommendationMethod",
    "Policy",
    "GroupPolicy",
    "LayerNorm",
    "Task",
    "Framework",
    "Framework",
    "Extension",
    "OriginFrom",
    "DataType",
    "DeviceName",
    "SoftwareVersion",
    "HardwareType",
    "TaskStatus",
    "Module",
    "Options",
    "ConvertAction",
    "ExperimentAction",
    "StepOp",
]



from pathlib import Path
from typing import Dict, List, Optional, Union
from urllib import request

from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.compressor import compressor_client
from netspresso.clients.compressor.schemas.compression import (
    AutoCompressionRequest,
    AvailableLayer,
    CompressionRequest,
    CreateCompressionRequest,
    GetAvailableLayersRequest,
    Options,
    RecommendationRequest,
    UploadDatasetRequest,
)
from netspresso.clients.compressor.schemas.model import UploadModelRequest
from netspresso.clients.launcher import launcher_client
from netspresso.compressor.core.compression import CompressionInfo
from netspresso.compressor.core.model import CompressedModel, Model, ModelCollection, ModelFactory
from netspresso.enums import CompressionMethod, Framework, Module, RecommendationMethod, ServiceCredit, Status, TaskType

from ..utils import FileHandler, check_credit_balance
from ..utils.metadata import MetadataHandler
from .utils.onnx import export_onnx


class Compressor:
    def __init__(self, token_handler: TokenHandler) -> None:
        """Initialize the Compressor."""

        self.token_handler = token_handler
        self.model_factory = ModelFactory()

    def upload_model(
        self,
        input_model_path: str,
        input_shapes: List[Dict[str, int]] = None,
        framework: Framework = Framework.PYTORCH,
    ) -> Model:
        """Upload a model for compression.

        Args:
            input_model_path (str): The file path where the model is located.
            input_shapes (List[Dict[str, int]], optional): Input shapes of the model. Defaults to [].
            framework (Framework): The framework of the model.

        Raises:
            e: If an error occurs while uploading the model.

        Returns:
            Model: Uploaded model object.
        """

        if input_shapes is None:
            input_shapes = []
        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        try:
            logger.info("Uploading Model...")

            model_name = Path(input_model_path).stem

            data = UploadModelRequest(
                model_name=model_name,
                framework=framework,
                file_path=input_model_path,
                input_layers=input_shapes,
            )
            model_info = compressor_client.upload_model(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            model = self.model_factory.create_model(model_info=model_info)

            logger.info(f"Upload model successfully. Model ID: {model.model_id}")

            return model

        except Exception as e:
            logger.error(f"Upload model failed. Error: {e}")
            raise e

    def get_model(self, model_id: str) -> Union[Model, CompressedModel]:
        """Get the model for a given model ID.

        Args:
            model_id (str): The ID of the model.

        Raises:
            e: If an error occurs while getting the model.

        Returns:
            Union[Model, CompressedModel]: The retrieved model. If the model is compressed,
            `CompressedModel` will be returned. Otherwise, `Model` will be returned.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Getting model...")
            model_info = compressor_client.get_model_info(
                model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            if model_info.status.is_compressed:
                model = self.model_factory.create_compressed_model(model_info=model_info)
            else:
                model = self.model_factory.create_model(model_info=model_info)
            logger.info("Get model successfully.")

            return model

        except Exception as e:
            logger.error(f"Get model failed. Error: {e}")
            raise e

    def download_model(self, model_id: str, local_path: str) -> None:
        """Download the model for a given model ID to the local path.

        Args:
            model_id (str): The ID of the model.
            local_path (str): The local path to save the downloaded model.

        Raises:
            e: If an error occurs while downloading the model.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Downloading model...")
            download_link = compressor_client.get_download_model_link(
                model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            request.urlretrieve(download_link.url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download model failed. Error: {e}")
            raise e

    def delete_model(self, model_id: str, recursive: bool = False) -> None:
        """Delete the model for a given model ID.

        Args:
            model_id (str): The ID of the model.
            recursive (bool, optional): Whether to also delete the compressed model for that model. Defaults to False.

        Raises:
            e: If an error occurs while deleting the model.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Deleting model...")
            children_models = compressor_client.get_children_models(
                model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            if len(children_models) != 0:
                if not recursive:
                    logger.warning(
                        "Deleting the model will also delete its compressed models. To proceed with the deletion, set the `recursive` parameter to True."
                    )
                else:
                    logger.info("The compressed model for that model will also be deleted.")
                    compressor_client.delete_model(
                        model_id=model_id,
                        access_token=self.token_handler.tokens.access_token,
                        verify_ssl=self.token_handler.verify_ssl,
                    )
                    logger.info("Delete model successfully.")
            else:
                logger.info("The model will be deleted.")
                compressor_client.delete_model(
                    model_id=model_id,
                    access_token=self.token_handler.tokens.access_token,
                    verify_ssl=self.token_handler.verify_ssl,
                )
                logger.info("Delete model successfully.")

        except Exception as e:
            logger.error(f"Delete model failed. Error: {e}")
            raise e

    def select_compression_method(
        self,
        model_id: str,
        compression_method: CompressionMethod,
        options: Options = Options(),
    ) -> CompressionInfo:
        """Select a compression method for a model.

        Args:
            model_id (str): The ID of the model.
            compression_method (CompressionMethod): The selected compression method.
            options(Options, optional): The options for pruning method.

        Raises:
            e: If an error occurs while selecting the compression method.

        Returns:
            CompressionInfo: The compression information for the selected compression method.
        """

        self.token_handler.validate_token()

        try:
            model = self.get_model(model_id)

            logger.info("Selecting compression method...")
            if model.framework == Framework.PYTORCH and compression_method == CompressionMethod.PR_NN:
                raise Exception("The Nuclear Norm is only supported in the TensorFlow-Keras framework.")

            data = GetAvailableLayersRequest(
                model_id=model.model_id,
                compression_method=compression_method,
                options=options,
            )
            response = compressor_client.get_available_layers(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            compression_info = CompressionInfo(
                original_model_id=model.model_id,
                compression_method=compression_method,
                options=options.dict(),
            )
            compression_info.set_available_layers(response.available_layers)
            logger.info("Select compression method successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Select compression method failed. Error: {e}")
            raise e

    def get_compression(self, compression_id: str) -> CompressionInfo:
        """Get information about a compression.

        Args:
            compression_id (str): The ID of the compression.

        Raises:
            e: If an error occurs while getting the compression information.

        Returns:
            CompressionInfo: The information about the compression.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Getting compression...")
            _compression_info = compressor_client.get_compression_info(
                compression_id=compression_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            compression_info = CompressionInfo(
                compressed_model_id=_compression_info.new_model_id,
                compression_id=_compression_info.compression_id,
                compression_method=_compression_info.compression_method,
            )
            compression_info.set_available_layers(_compression_info.available_layers)
            logger.info("Get compression successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Get compression failed. Error: {e}")
            raise e

    def __upload_dataset(self, model_id: str, dataset_path: str) -> None:
        """Upload a dataset for nuclear norm compression method.

        Args:
            model_id (str): The ID of the model.
            dataset_path (str): The file path where the dataset is located.

        Raises:
            e: If an error occurs while uploading the dataset.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Uploading dataset...")
            data = UploadDatasetRequest(model_id=model_id, file_path=dataset_path)
            compressor_client.upload_dataset(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info("Upload dataset successfully.")

        except Exception as e:
            logger.error(f"Upload dataset failed. Error: {e}")
            raise e

    def _get_available_devices(self, compressed_model, default_model_path: str):
        """Get the available devices for the compressed model.

        Args:
            compressed_model: The compressed model.
            default_model_path (str): Path to the default model file.

        Returns:
            str: The uploaded model after conversion.
        """

        if compressed_model.framework in [Framework.PYTORCH, Framework.ONNX]:
            export_onnx(default_model_path, compressed_model.input_shapes)
            converter_uploaded_model = launcher_client.upload_model(
                model_file_path=default_model_path.with_suffix(".onnx"),
                target_function=Module.CONVERT,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
        else:
            converter_uploaded_model = launcher_client.upload_model(
                model_file_path=default_model_path.with_suffix(".h5"),
                target_function=Module.CONVERT,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

        return converter_uploaded_model

    def compress_model(
        self,
        compression: CompressionInfo,
        output_dir: str,
        dataset_path: Optional[str] = None,
    ) -> Dict:
        """Compress a model using the provided compression information.

        Args:
            compression (CompressionInfo): The information about the compression.
            output_dir (str): The local path to save the compressed model.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while compressing the model.

        Returns:
            Dict: Source model and compressed model information.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Compressing model...")

            model_info = self.get_model(compression.original_model_id)

            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=model_info.framework
            )
            FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = MetadataHandler.init_metadata(folder_path=output_dir, task_type=TaskType.COMPRESS)

            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            check_credit_balance(
                user_credit=current_credit,
                service_credit=ServiceCredit.ADVANCED_COMPRESSION,
            )

            model_name = Path(output_dir).name

            data = CreateCompressionRequest(
                model_id=compression.original_model_id,
                model_name=model_name,
                compression_method=compression.compression_method,
                options=compression.options,
            )
            compression_info = compressor_client.create_compression(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )

            if dataset_path and compression.compression_method == CompressionMethod.PR_NN:
                self.__upload_dataset(model_id=compression.original_model_id, dataset_path=dataset_path)

            for available_layers in compression.available_layers:
                if available_layers.values != [""]:
                    available_layers.use = True

            all_layers_false = all(available_layer.values == [""] for available_layer in compression.available_layers)
            if all_layers_false:
                raise Exception(
                    "The available_layer.values all empty. please put in the available_layer.values to compress."
                )

            available_layers = [
                AvailableLayer(
                    name=layer.name,
                    values=layer.values,
                    channels=layer.channels,
                    use=layer.use,
                )
                for layer in compression.available_layers
            ]

            data = CompressionRequest(
                compression_id=compression_info.compression_id,
                compression_method=compression.compression_method,
                layers=available_layers,
                compressed_model_id=compression_info.new_model_id,
                options=compression.options,
            )
            compressor_client.compress_model(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )

            self.download_model(
                model_id=compression_info.new_model_id,
                local_path=default_model_path.with_suffix(extension),
            )
            compressed_model = self.get_model(model_id=compression_info.new_model_id)

            converter_uploaded_model = self._get_available_devices(compressed_model, default_model_path)

            logger.info(f"Compress model successfully. Compressed Model ID: {compressed_model.model_id}")
            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.ADVANCED_COMPRESSION} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

            metadata.update_compressed_model_path(
                compressed_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            if compressed_model.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(
                    compressed_onnx_model_path=default_model_path.with_suffix(".onnx").as_posix()
                )
            metadata.update_model_info(
                task=model_info.task,
                framework=model_info.framework,
                input_shapes=model_info.input_shapes,
            )
            metadata.update_compression_info(
                method=compression.compression_method,
                options=compression.options,
                layers=compression.available_layers,
            )
            metadata.update_results(model=model_info, compressed_model=compressed_model)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_devices(converter_uploaded_model.available_devices)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return metadata.asdict()

        except Exception as e:
            logger.error(f"Compress model failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def recommendation_compression(
        self,
        compression_method: CompressionMethod,
        recommendation_method: RecommendationMethod,
        recommendation_ratio: float,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        options: Options = Options(),
        dataset_path: Optional[str] = None,
    ) -> Dict:
        """Compress a recommendation-based model using the given compression and recommendation methods.

        Args:
            compression_method (CompressionMethod): The selected compression method.
            recommendation_method (RecommendationMethod): The selected recommendation method.
            recommendation_ratio (float): The compression ratio recommended by the recommendation method.
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local path to save the compressed model.
            input_shapes (List[Dict[str, int]]): Input shapes of the model.
            framework (Framework, optional): The framework of the model.
            options(Options, optional): The options for pruning method.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while performing recommendation compression.

        Returns:
            Dict: Source model and compressed model information.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        try:
            logger.info("Compressing recommendation-based model...")

            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=framework
            )
            FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = MetadataHandler.init_metadata(folder_path=output_dir, task_type=TaskType.COMPRESS)

            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            check_credit_balance(
                user_credit=current_credit,
                service_credit=ServiceCredit.ADVANCED_COMPRESSION,
            )

            if framework == Framework.PYTORCH and compression_method == CompressionMethod.PR_NN:
                raise Exception("The Nuclear Norm is only supported in the TensorFlow-Keras framework.")

            if compression_method in [CompressionMethod.PR_ID, CompressionMethod.FD_CP]:
                raise Exception(
                    f"The {compression_method} compression method you choose doesn't provide a recommendation."
                )

            if (
                compression_method
                in [
                    CompressionMethod.PR_L2,
                    CompressionMethod.PR_GM,
                    CompressionMethod.PR_NN,
                ]
                and recommendation_method != RecommendationMethod.SLAMP
            ):
                raise Exception(
                    f"The {compression_method} compression method is only available the SLAMP recommendation method."
                )

            if (
                compression_method in [CompressionMethod.FD_TK, CompressionMethod.FD_SVD]
                and recommendation_method != RecommendationMethod.VBMF
            ):
                raise Exception(
                    f"The {compression_method} compression method is only available the VBMF recommendation method."
                )

            model_name = Path(output_dir).name

            model = self.upload_model(
                framework=framework,
                input_model_path=input_model_path,
                input_shapes=input_shapes,
            )

            data = CreateCompressionRequest(
                model_id=model.model_id,
                model_name=model_name,
                compression_method=compression_method,
                options=options.dict(),
            )
            compression_info = compressor_client.create_compression(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )

            if dataset_path and compression_method == CompressionMethod.PR_NN:
                self.__upload_dataset(model_id=model.model_id, dataset_path=dataset_path)

            data = RecommendationRequest(
                model_id=model.model_id,
                compression_id=compression_info.compression_id,
                recommendation_method=recommendation_method,
                recommendation_ratio=recommendation_ratio,
                options=options.dict(),
            )
            logger.info("Compressing model...")
            recommendation_result = compressor_client.get_recommendation(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )

            for recommended_layer in recommendation_result.recommended_layers:
                for available_layer in compression_info.available_layers:
                    # Find the matching available_layer by name
                    if available_layer.name == recommended_layer.name:
                        available_layer.use = True
                        available_layer.values = recommended_layer.values

            data = CompressionRequest(
                compression_id=compression_info.compression_id,
                compression_method=compression_method,
                layers=compression_info.available_layers,
                compressed_model_id=compression_info.new_model_id,
                options=options.dict(),
            )
            compressor_client.compress_model(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )

            self.download_model(
                model_id=compression_info.new_model_id,
                local_path=default_model_path.with_suffix(extension),
            )
            compressed_model = self.get_model(model_id=compression_info.new_model_id)

            converter_uploaded_model = self._get_available_devices(compressed_model, default_model_path)

            logger.info(f"Recommendation compression successfully. Compressed Model ID: {compressed_model.model_id}")
            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.ADVANCED_COMPRESSION} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

            _compression_info = self.get_compression(compression_info.compression_id)
            metadata.update_compressed_model_path(
                compressed_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            if compressed_model.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(
                    compressed_onnx_model_path=default_model_path.with_suffix(".onnx").as_posix()
                )
            metadata.update_model_info(task=model.task, framework=framework, input_shapes=input_shapes)
            metadata.update_compression_info(
                method=_compression_info.compression_method,
                ratio=recommendation_ratio,
                options=_compression_info.options,
                layers=_compression_info.available_layers,
            )
            metadata.update_results(model=model, compressed_model=compressed_model)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_devices(converter_uploaded_model.available_devices)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return metadata.asdict()

        except Exception as e:
            logger.error(f"Recommendation compression failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def automatic_compression(
        self,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        compression_ratio: float = 0.5,
    ) -> Dict:
        """Compress a model automatically based on the given compression ratio.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local path to save the compressed model.
            input_shapes (List[Dict[str, int]]): Input shapes of the model.
            framework (Framework, optional): The framework of the model.
            compression_ratio (float, optional): The compression ratio for automatic compression. Defaults to 0.5.

        Raises:
            e: If an error occurs while performing automatic compression.

        Returns:
            Dict: Source model and compressed model information.
        """

        FileHandler.check_input_model_path(input_model_path)

        self.token_handler.validate_token()

        try:
            logger.info("Compressing automatic-based model...")

            default_model_path, extension = FileHandler.get_path_and_extension(
                folder_path=output_dir, framework=framework
            )
            FileHandler.create_unique_folder(folder_path=output_dir)
            metadata = MetadataHandler.init_metadata(folder_path=output_dir, task_type=TaskType.COMPRESS)

            current_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            check_credit_balance(
                user_credit=current_credit,
                service_credit=ServiceCredit.AUTOMATIC_COMPRESSION,
            )

            model_name = Path(output_dir).name

            model = self.upload_model(
                framework=framework,
                input_model_path=input_model_path,
                input_shapes=input_shapes,
            )

            compressed_model_name = f"{model_name}_automatic_{compression_ratio}"
            data = AutoCompressionRequest(
                model_id=model.model_id,
                model_name=compressed_model_name,
                recommendation_ratio=compression_ratio,
                save_path=output_dir,
            )
            logger.info("Compressing model...")
            model_info = compressor_client.auto_compression(
                data=data, access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            compression_info = self.get_compression(model_info.original_compression_id)

            self.download_model(
                model_id=model_info.model_id,
                local_path=default_model_path.with_suffix(extension),
            )
            compressed_model = self.model_factory.create_compressed_model(model_info=model_info)  # TODO: delete

            converter_uploaded_model = self._get_available_devices(compressed_model, default_model_path)

            logger.info(f"Automatic compression successfully. Compressed Model ID: {compressed_model.model_id}")
            remaining_credit = auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(
                f"{ServiceCredit.AUTOMATIC_COMPRESSION} credits have been consumed. Remaining Credit: {remaining_credit}"
            )

            metadata.update_compressed_model_path(
                compressed_model_path=default_model_path.with_suffix(extension).as_posix()
            )
            if compressed_model.framework in [Framework.PYTORCH, Framework.ONNX]:
                metadata.update_compressed_onnx_model_path(
                    compressed_onnx_model_path=default_model_path.with_suffix(".onnx").as_posix()
                )
            metadata.update_model_info(task=model.task, framework=framework, input_shapes=input_shapes)
            metadata.update_compression_info(
                method=compression_info.compression_method,
                ratio=compression_ratio,
                options=compression_info.options,
                layers=compression_info.available_layers,
            )
            metadata.update_results(model=model, compressed_model=compressed_model)
            metadata.update_status(status=Status.COMPLETED)
            metadata.update_available_devices(converter_uploaded_model.available_devices)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return metadata.asdict()

        except Exception as e:
            logger.error(f"Automatic compression failed. Error: {e}")
            metadata.update_status(status=Status.ERROR)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)
            raise e

        except KeyboardInterrupt:
            metadata.update_status(status=Status.STOPPED)
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)



from netspresso_trainer.cfg import ClassificationScheduleConfig, DetectionScheduleConfig, SegmentationScheduleConfig

TRAINING_CONFIG_TYPE = {
    "classification": ClassificationScheduleConfig,
    "detection": DetectionScheduleConfig,
    "segmentation": SegmentationScheduleConfig,
}
